
#-----------------------------------------------------------------------------
# MACRO TO ADD A COMPILER DEFINITION WHICH ADDS ALSO THE DEF 
# INTO DOXYGEN PREDEFINED VAR 
# (OTHERWISE DOXYGEN WOULD NOT PARSE THE FILES BLOCKED)
MACRO(ADD_DEF DEF)
  MESSAGE(STATUS "* Added definition -D${DEF}")
  ADD_DEFINITIONS( -D${DEF} )
  SET(DOXYGEN_DOC_PREDEFINED 
    ${DOXYGEN_DOC_PREDEFINED};${DEF}
    CACHE 
    STRING 
    ""
    FORCE)
  MARK_AS_ADVANCED(DOXYGEN_DOC_PREDEFINED)
ENDMACRO(ADD_DEF)
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# MACRO TO ADD A COMPILER DEFINITION WITH VALUE WHICH ADDS ALSO THE DEF 
# INTO DOXYGEN PREDEFINED VAR 
# (OTHERWISE DOXYGEN WOULD NOT PARSE THE FILES BLOCKED)
MACRO(ADD_DEF_WITH_VAL DEF VAL)
  MESSAGE(STATUS "* Added definition -D${DEF}=${VAL}")
  ADD_DEFINITIONS( -D${DEF}=${VAL} )
  SET(DOXYGEN_DOC_PREDEFINED 
    ${DOXYGEN_DOC_PREDEFINED};${DEF}=${VAL}
    CACHE 
    STRING 
    ""
    FORCE)
  MARK_AS_ADVANCED(DOXYGEN_DOC_PREDEFINED)
ENDMACRO(ADD_DEF_WITH_VAL)
#-----------------------------------------------------------------------------


#========================================================================
# MACRO WHICH COPIES ALL IMAGES (png, jpg, gif) 
# FROM SOURCE TREE TO DOC BUILD TREE
# AND INSTALL THEM IN BBTK DOC TREE
MACRO(BBTK_DOC_INSTALL_IMAGES DOC_RELATIVE_INSTALL_PATH)
  MESSAGE(STATUS "* Copying png,jpg,tif images in ${BBTK_DOC_BUILD_PATH}/${DOC_RELATIVE_INSTALL_PATH}")
  FILE(GLOB PNG_IMAGES "*.png")
  FILE(GLOB JPG_IMAGES "*.jpg")
  FILE(GLOB TIF_IMAGES "*.tif")
  SET(IMAGES
    ${PNG_IMAGES}
    ${JPG_IMAGES}
    ${TIF_IMAGES}
    )
#    MESSAGE(ERROR ${IMAGES} )
  FOREACH(image ${IMAGES})  
    GET_FILENAME_COMPONENT(filename "${image}" NAME)
    CONFIGURE_FILE(
      ${image}
       ${BBTK_DOC_BUILD_PATH}/${DOC_RELATIVE_INSTALL_PATH}/${filename}
      COPYONLY
      ) 
      
    #  IF (WIN32)
#	CONFIGURE_FILE(
#          ${image}
#          ${CMAKE_CURRENT_BINARY_DIR}/${filename}
#          COPYONLY
#	  )
#      ENDIF(WIN32)
    # MESSAGE(ERROR "${image} (${filename}) in :  ${PROJECT_BINARY_DIR}/${BBTK_DOC_INSTALL_PATH}/${DOC_RELATIVE_INSTALL_PATH}/${filename}")
  ENDFOREACH(image)
  INSTALL( 
      FILES ${IMAGES} 
      DESTINATION ${BBTK_DOC_INSTALL_PATH}/${DOC_RELATIVE_INSTALL_PATH}
    )  
ENDMACRO(BBTK_DOC_INSTALL_IMAGES)
#========================================================================

#========================================================================
# Creates a directory if it does not exist
MACRO(MKDIR DIR)
  IF(NOT IS_DIRECTORY ${DIR})
    MESSAGE(STATUS "* Creating directory '${DIR}'")
    FILE(MAKE_DIRECTORY ${DIR})
  ENDIF(NOT IS_DIRECTORY ${DIR})
ENDMACRO(MKDIR)
#========================================================================


#========================================================================
MACRO(MAKE_DLL_PATH)
  IF(WIN32)
    IF(USE_DOT)
      IF (NOT DOT_DIR)
        FIND_PATH(DOT_DIR dot.exe PATHS $ENV{PATH} )
        IF(NOT DOT_DIR)
          MESSAGE(FATAL_ERROR "Dot not found : set DOT_DIR to the folder containing dot.exe")
        ELSE(NOT DOT_DIR)
          MESSAGE(STATUS "Looking for dot... - found in ${DOT_DIR}")
         ENDIF(NOT DOT_DIR)
      ENDIF(NOT DOT_DIR)
    ENDIF(USE_DOT)	
#    SET(DLL_PATHA Path=${ADDITIONAL_DLL_PATH}__RR__%WXWIN%/lib/vc_dll/__RR__%VTK_DIR%__RR__%ITK_DIR%/../../bin/__RR__%PATH%__RR__${DOT_DIR})
    SET(DLL_PATHA ${ADDITIONAL_DLL_PATH}__RR__${DOT_DIR}__RR__)
    FOREACH(APATH $ENV{PATH})
	  SET(DLL_PATHA ${DLL_PATHA}__RR__${APATH})
    ENDFOREACH(APATH)
    STRING(REPLACE "__RR__" "\;" DLL_PATH ${DLL_PATHA} )
  ENDIF(WIN32)  
ENDMACRO(MAKE_DLL_PATH)
#========================================================================
