/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkBlackBoxOutputConnector.cxx,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:12 $
  Version:   $Revision: 1.8 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 *  \file 
 *  \brief Class bbtk::BlackBoxOutputConnector : 
 */

#include "bbtkBlackBoxOutputConnector.h"
#include "bbtkMessageManager.h"
#include "bbtkBlackBox.h"
//#include <algorithm>

namespace bbtk
{

  BlackBoxOutputConnector::BlackBoxOutputConnector() 
  // : mStatus(MODIFIED)
  {
    bbtkDebugMessage("Kernel",9,"BlackBoxOutputConnector::BlackBoxOutputConnector()"<<std::endl);
  }

  BlackBoxOutputConnector::~BlackBoxOutputConnector() 
  {
    bbtkDebugMessage("Kernel",9,
		     "==> BlackBoxOutputConnector::~BlackBoxOutputConnector()"
		     <<std::endl);
    /*
    ConnectionVector::iterator i;
    for (i=mConnection.begin();
	 i!=mConnection.end();
	 ++i)
      (*i) = Connection::WeakPointer();
    */
    mConnection.clear();
    bbtkDebugMessage("Kernel",9,
		     "<== BlackBoxOutputConnector::~BlackBoxOutputConnector()"
		     <<std::endl);
  }

  ///
  void BlackBoxOutputConnector::SetConnection(Connection* c)
  {
    bbtkDebugMessage("Kernel",9,"BlackBoxOutputConnector::SetConnection("
		     <<c<<")"<<std::endl);
    mConnection.push_back(c);
  }



  ///
  void BlackBoxOutputConnector::UnsetConnection(Connection* c)
  {
    bbtkDebugMessageInc("Kernel",9,"BlackBoxOutputConnector::UnsetConnection("
		     <<c<<")"<<std::endl);

    if (!c) 
      {
	bbtkInternalError("BlackBoxOutputConnector::UnsetConnection("<<c<<") : invalid connection");

      }
    //    c->Check();
    //Connection::WeakPointer w(c);
    ConnectionVector::iterator i;
    //  = find(mConnection.begin(),mConnection.end(),c);
    for (i=mConnection.begin();
	 i!=mConnection.end();
	 ++i)
      {
	if (*i==c) break;
      }
    if (i==mConnection.end())
      {
	bbtkInternalError("BlackBoxOutputConnector::UnsetConnection("<<c<<") : connection is absent from connections list");
      }
    mConnection.erase(i);

    bbtkDebugDecTab("Kernel",9);
  }


  
  void BlackBoxOutputConnector::SetModifiedStatus()
  {
    bbtkDebugMessage("modified",2,
		     "==> BlackBoxOutputConnector::SetModifiedStatus() ["
		     <<this<<"]"
		     <<std::endl);
    ConnectionVector::iterator i;
    for (i=mConnection.begin();i!=mConnection.end();++i) 
      {
	(*i)->SetModifiedStatus();
      }
    
    
   }
 


}
// namespace bbtk

