/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbvtkImagePlanes.h,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:30 $
  Version:   $Revision: 1.6 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 * \brief Short description in one line
 * 
 * Long description which 
 * can span multiple lines
 */
/**
 * \file 
 * \brief Pattern for the definition of a new type of Node (header)
 */
/**
 * \class bbtk::NodePatern 
 * \brief Pattern for the definition of a new type of Node 
 */


#ifdef _USE_VTK_


#ifndef __bbvtkImagePlanes_h_INCLUDED__
#define __bbvtkImagePlanes_h_INCLUDED__


#include "bbtkAtomicBlackBox.h"

#include "vtkImageData.h"
//#include "vtkOutlineFilter.h"
//#include "vtkPolyDataMapper.h"
//#include "vtkActor.h"
#include "vtkImagePlaneWidget.h"
#include "vtkCellPicker.h"
//#include "vtkProperty.h"

//#include "vtkRenderer.h"
//#include "vtkCamera.h"

namespace bbvtk
{

  typedef std::vector<double> winLevel;
  
  class /*BBTK_EXPORT*/ ImagePlanes : public bbtk::AtomicBlackBox
  { 
  public:
    BBTK_BLACK_BOX_INTERFACE(ImagePlanes,bbtk::AtomicBlackBox);
    //    BBTK_DECLARE_INPUT(Contour,int);
    BBTK_DECLARE_INPUT(In,vtkImageData *);
    BBTK_DECLARE_INPUT(WindowLevel,winLevel);
    BBTK_DECLARE_OUTPUT(PlaneX,vtkImagePlaneWidget*);
    BBTK_DECLARE_OUTPUT(PlaneY,vtkImagePlaneWidget*);
    BBTK_DECLARE_OUTPUT(PlaneZ,vtkImagePlaneWidget*);
    BBTK_PROCESS(Process);
    void Process();
    
  protected:
    virtual void bbUserConstructor();
    virtual void bbUserCopyConstructor();
    virtual void bbUserDestructor();
    void Init();
  private:
    vtkImagePlaneWidget* planeWidgetX;
    vtkImagePlaneWidget* planeWidgetY;
    vtkImagePlaneWidget* planeWidgetZ;
  };
   
  //=================================================================
  // UserBlackBox description
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(ImagePlanes,bbtk::AtomicBlackBox);
  BBTK_NAME("ImagePlanes");
  BBTK_AUTHOR("laurent.guigues@creatis.insa-lyon.fr");
  BBTK_DESCRIPTION("Creates three 3D planes with the input image mapped onto with which the user can interact. The output vtkImagePlaneWidget objects are to be inserted into a 3D scene (e.g. a Viewer3D)");
  BBTK_CATEGORY("3D object creator");
  BBTK_INPUT(ImagePlanes,In,"Input Image",vtkImageData *,"");
  BBTK_INPUT(ImagePlanes,WindowLevel,"Widdow Level",winLevel,"");  
  BBTK_OUTPUT(ImagePlanes,PlaneX,"The image plane in X direction",vtkImagePlaneWidget*,"");
  BBTK_OUTPUT(ImagePlanes,PlaneY,"The image plane in Y direction",vtkImagePlaneWidget*,"");
  BBTK_OUTPUT(ImagePlanes,PlaneZ,"The image plane in Z direction",vtkImagePlaneWidget*,"");
  BBTK_END_DESCRIBE_BLACK_BOX(ImagePlanes);
  //=================================================================





}//namespace bbvtk

#endif  //__bbtvtkImagePlanes_h__

#endif //_USE_VTK_

